#ifndef VISITE_H
#define VISITE_H

#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include "coda.h"
#include "lista.h"
#include "pila.h"

// srtuttura che mi serve per la visita con tempi di inizio/fine visita
typedef struct VISITATI {
	int invis;
	int finvis;
} visitati;

// struttura che serve per l'implementazione dell'algoritmo di Prim versione O(n^2)
typedef struct VISITATI_PRIM {
	int visitato;
	int costo;
	int radice;
} visitati_prim;

// struttura che mi serve per l'implementazione dell'algoritmo di Kruskal
typedef struct KRUSKAL {
	int radice;
	int rango;
} kruskal;

// struttura generica per memorizzare array di spigoli e relativi pesi
typedef struct SPIGOLI {
	int vertice1;
	int vertice2;
	int costo;
} spigoli;



// NOTA: le due funzioni sulle visite richiedono un vettore per le verifiche GIA' allocato a n_nodi+1 e azzerato opportunamente 
void DFS(list_Ptr *L, int nodo, int n_nodi, int *vis);
void DFS2(list_Ptr *L, int nodo, int n_nodi, visitati *vis);
/* in questa versione la dfs scrive i tempi di inizio e fine visita sul vettore di tipo "visitati" */
void DFS_INFIN(list_Ptr *L, int nodo, int n_nodi, visitati *VIS, int *i);
// questa e' una versione RICORSIVA della dfs che scrive anche i numeri di inizio e fine visita
// la prima volta va chiamata con un VIS gia' inizializzato e cn *i = 1
	
// il numero dei nodi va passato non incrementato di 1. ci pensa la funzione
void BFS(list_Ptr *L, int nodo, int n_nodi, int *vis);
// il numero dei nodi va passato non incrementato di 1. ci pensa la funzione
list_Ptr* ordina_lista(list_Ptr *L, int n_nodi);
// stesso discorso di sopra. Aggiunge con un addhead quindi il nuovo grafo risulta ordinato in ordine decrescente

int compConn(list_Ptr *L, int nodo, int n_nodi);
// usa la BFS e il numero dei nodi va passato non incrementato di 1

list_Ptr* costruisci(list_Ptr *L, int n_nodi);
// il numero dei nodi va passato non incrementato di 1. ci pensa la funzione

int *calcola_ordine(list_Ptr *L, int n_nodi);
list_Ptr rel_ordine(list_Ptr *L, int n_nodi);

int* compfortconn(list_Ptr *G, int n_nodi, int nodo);
// ritorna una matrice con i nomi dei vertici che fanno parte delle componenti f. connesse 
void DFS_INFIN_ELIM(list_Ptr *L, int nodo, int n_nodi, visitati *VIS, int *i);
// fa la DFS ma elimina anche i nodi visitati
	
// codice di Prufer
int* PruferOLD(list_Ptr *G, int n_nodi);
// il vettore che ritorna e' di 1 piu' piccolo di n_nodi (proprieta' del codice di Prufer)
//
// NOTA! in teoria l'algoritmo che genera il codice di Prufer dovrebbe lavorare su una lista di adiacenza che e' quella
// originale del grafo e su un'altra lista di adiacenza che invece rappresenta il grafo radicato (per rappresentare
// un albero n-ario). Questa cosa e' necessaria perche' altrimenti quando vado ad esaminare una foglia, se avessi solo il grafo
// radicato non potrei sapere a QUALE elemento del grafo devo andare a decrementare il grado (di uscita, in questo caso),
// perche', trattandosi proprio di una foglia e trovandoci di fronte ad un grafo orientato, questo tipo di informazione non 
// ce l'ho.
// Per evitare di dover passare troppe cose a questa funzione io in verita' lavoro direttamente sul grafo senza utilizzare
// la lista di adiacenza che rappresenta il grafo radicato. Per questo quando faccio i controlli sul grado, controllo che 
// questo sia 1 e non zero, perche' nel mio caso un elemento e' foglia quando ha grado 1, non 0 (cosa che invece accadrebbe
// nel caso del grafo orientato) OK?

list_Ptr* De_PruferOLD(int *codice, int n_nodi);
// nn mi funziona (AZZZZ)

list_Ptr* Gq(list_Ptr *G, int n_nodi); 
// genera G^2 a partire da G.

int* Prufer(list_Ptr *G, int n_nodi);
// versione ottima presa dalle dispense
int* De_Prufer(int *codice, int n_nodi);
// versione ottima presa dalle dispense


/* ALGORITMI DI KRUSKAL E PRIM */
list_Ptr addHead_pes(list_Ptr L, int elem, int cost);
int find_min_prim(visitati_prim *V, int n_elem);
list_Ptr *costruisci_pes(list_Ptr *L, int n_nodi);

// prim versione n^2
list_Ptr* Prim_n2(list_Ptr *G, int n_nodi, int nodo_iniziale);

// prim versione mlog(m)
void heapify_min_prim(int *heap, int in, int fin, int *costo);
void insheap_min_prim(int *H, int i, int *costo);
int extractheap_min_prim(int *H, int n_elem, int *costo); 
list_Ptr* Prim_mlogm(list_Ptr *G, int n_nodi, int nodo_iniziale);

// kruskal
spigoli* crea_lista_spigoli(int *n);
int cerca_radice(kruskal *K, int elem);
list_Ptr* Kruskal(spigoli *S, int n_elem, int n_spigoli);

#endif
